@@ -132,8 +132,12 @@ class AdvancedHTMLParser(HTMLParser):
         '''
             Internal for parsing
         '''
-        if data and len(self.inTag) > 0:
-            self.inTag[-1].appendText(data)
+        if data:
+            if len(self.inTag) > 0:
+                self.inTag[-1].appendText(data)
+            elif data.strip(): #and not self.getRoot():
+                # Must be text prior to or after root node
+                raise MultipleRootNodeException()
 
     def handle_entityref(self, entity):
         '''
@@ -141,6 +145,8 @@ class AdvancedHTMLParser(HTMLParser):
         '''
         if len(self.inTag) > 0:
             self.inTag[-1].appendText('&%s;' %(entity,))
+        else:
+            raise MultipleRootNodeException()
 
     def handle_charref(self, charRef):
         '''
@@ -148,6 +154,8 @@ class AdvancedHTMLParser(HTMLParser):
         '''
         if len(self.inTag) > 0:
             self.inTag[-1].appendText('&#%s;' %(charRef,))
+        else:
+            raise MultipleRootNodeException()
 
     def handle_comment(self, comment):
         '''
@@ -155,6 +163,8 @@ class AdvancedHTMLParser(HTMLParser):
         '''
         if len(self.inTag) > 0:
             self.inTag[-1].appendText('<!-- %s -->' %(comment,))
+        else:
+            raise MultipleRootNodeException()
 
     def handle_decl(self, decl):
         '''
@@ -359,7 +369,15 @@ class AdvancedHTMLParser(HTMLParser):
         else:
             doctypeStr = ''
 
-        return doctypeStr + ''.join([elem.outerHTML for elem in self.getRootNodes()])
+        # 6.6.0: If we have a real root tag, print the outerHTML. If we have a fake root tag (for multiple root condition),
+        #   then print the innerHTML (skipping the outer root tag). Otherwise, we will miss
+        #   untagged text (between the multiple root nodes).
+        rootNode = self.getRoot()
+        if rootNode.tagName == INVISIBLE_ROOT_TAG:
+            return doctypeStr + rootNode.innerHTML
+        else:
+            return doctypeStr + rootNode.outerHTML
+#        return doctypeStr + ''.join([elem.outerHTML for elem in self.getRootNodes()])
 
 
     def getFormattedHTML(self, indent='  '):
